//temporalRow.cxx
//
//Chris Bentzel, 01.16.98

/***************************************************************************
 * 
 * Copyright 1998  by  the  Massachusetts Institute  of Technology.   All  
 * rights reserved.
 * 
 * Developed  by the Perceptual Computing Section 
 * at the Media Laboratory, MIT, Cambridge, Massachusetts.
 * 
 * Permission to use, copy,  or modify this software and  its documentation
 * for  educational  and  research purposes only and without fee  is hereby
 * granted, provided  that this copyright notice and the original authors's
 * names appear  on all copies and supporting documentation.  If individual
 * files are  separated from  this  distribution directory  structure, this
 * copyright notice must be included.  For any other uses of this software,
 * in original or  modified form, including but not limited to distribution
 * in whole or in  part, specific  prior permission  must be  obtained from
 * MIT.  These programs shall not  be  used, rewritten, or  adapted as  the
 *   basis  of  a commercial  software  or  hardware product  without first
 * obtaining appropriate licenses  from MIT.  MIT. makes no representations
 * about the suitability of this  software for any purpose.  It is provided
 * "as is" without express or implied warranty.
 * 
 **************************************************************************/

#include "temporalRow.hxx"

void
temporalRow::getEntry(int index, temporalRowEntry& entryData) const throw(negativeIndexException, overflowIndexException)
{
  if (index < 0)
    {
      throw negativeIndexException("getEntry");
    }
  else
    {
      if (index >= numElems)
        {
          throw overflowIndexException("getEntry");
        }
      else
        {
          entryData = rep[index];  
        }
    }
}


void
temporalRow::setEntry(int index, temporalRowEntry& entryData) throw(negativeIndexException, overflowIndexException)
{
  if (index < 0 )
    {
      throw negativeIndexException("setEntry");
    }
  else
    {
      if (index >= numElems)
        {
          throw overflowIndexException("setEntry");
        }
      else
        {
          rep[index] = entryData;
        }
    }
}

void
temporalRow::setLatestEntry(temporalRowEntry& entryData) throw(negativeIndexException)
{
  if (numElems <= 0)
    {
      throw negativeIndexException("setLatestEntry");
    }
  //shift everything if placing in the last index
  if (latest == numElems -1)
    {
      shift();
    }
  //otherwise move it up
  else
    {
      latest++;
    }

  //copy the data
  rep[latest] = entryData;
}

void
temporalRow::getLatestEntry(temporalRowEntry& entryData) const throw(negativeIndexException)
{
  if (numElems <= 0 || latest < 0)
    {
      throw negativeIndexException("getLatestEntry");
    }
  entryData = rep[latest];
}

int
temporalRow::getSupplierIndex(void) const throw(negativeIndexException)
{
  if (sup < 0)
    throw negativeIndexException("getSupplierIndex");
  return sup;
}

void
temporalRow::setSupplierIndex(int index) throw(negativeIndexException, overflowIndexException)
{
  if (index < 0)
    throw negativeIndexException("setSupplierIndex");
  else
    sup = index;
}

void
temporalRow::setNumberElements(int numberElements) throw(negativeIndexException)
{
  if (numberElements < 0)
    throw negativeIndexException("setNumberElements");
  else
    {
      if (numberElements > numElems)
        {
          
          //allocate only once
          rep.reserve(numberElements);
          
          //add some elements
          for (int i = numElems; i < numberElements; i++)
            {
              temporalRowEntry ftang;
              rep.push_back(ftang);
            }

          //change the number of elements
          numElems = numberElements;
        }
    }
}







