// DriveTrainDoc.h : interface of the CDriveTrainDoc class
// Chris Bentzel

/***************************************************************************
 * 
 * Copyright 1996  by  the  Massachusetts Institute  of Technology.   All  
 * rights reserved.
 * 
 * Developed  by the Perceptual Computing Section 
 * at the Media Laboratory, MIT, Cambridge, Massachusetts.
 * 
 * Permission to use, copy,  or modify this software and  its documentation
 * for  educational  and  research purposes only and without fee  is hereby
 * granted, provided  that this copyright notice and the original authors's
 * names appear  on all copies and supporting documentation.  If individual
 * files are  separated from  this  distribution directory  structure, this
 * copyright notice must be included.  For any other uses of this software,
 * in original or  modified form, including but not limited to distribution
 * in whole or in  part, specific  prior permission  must be  obtained from
 * MIT.  These programs shall not  be  used, rewritten, or  adapted as  the
 *   basis  of  a commercial  software  or  hardware product  without first
 * obtaining appropriate licenses  from MIT.  MIT. makes no representations
 * about the suitability of this  software for any purpose.  It is provided
 * "as is" without express or implied warranty.
 * 
 **************************************************************************/

// BEGIN_IVWGEN
#include <Ivf/IvfDocument.h>
// END_IVWGEN
#include <Inventor/So.h>
#include "InsideTrakTransform.h"
#include "Quadraped.h"
#include "QuadrapedMap.h"
#include "GenMap.h"
#include "LinRBF.h"
#include "LinRBFAbs.h"
#include "CollectSequenceDialog.h"
#include "NewMapDialog.h"
#include "Model.h"
#include "SplineDialog.h"


#define NUM_MAPS 4
class CDriveTrainDialog;

// BEGIN_IVWGEN
class CDriveTrainDoc : public CDocument , public CIvfDocument
{
// END_IVWGEN
protected: // create from serialization only
	CDriveTrainDoc();
	DECLARE_DYNCREATE(CDriveTrainDoc)
	BOOLEAN m_bContainsCamera;

// Attributes
public:
	enum 
	{
	DRIVE_MODE,
	SNAPSHOT_MODE,
	SEQUENCE_MODE,
	SEQUENCE_WAIT_MODE,
	POSE_MODE,
	COLLECT_MODE,
	//YURI_MODE
	};

	static void InTrakSensorCB(void* data, SoSensor* sensor);

	//set get functions... all inlined
	int getCurrentState();
	void setCurrentState(int state);
	int getNumberOfTicks();
	void setNumberOfTicks(int ticks);
	int getMaxNumberOfTicks();
	void setMaxNumberOfTicks(int ticks);
	void setInTrakData(M<double>&);
	void setPdfInput(M<double>&);
;
// Operations
public:

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CDriveTrainDoc)
	public:
	virtual BOOL OnNewDocument();
	virtual BOOL OnOpenDocument(LPCTSTR lpszPathName);
	virtual void Serialize(CArchive& ar);
	virtual BOOL OnSaveDocument(LPCTSTR lpszPathName);
	//}}AFX_VIRTUAL

// Implementation
public:
	virtual ~CDriveTrainDoc();
	virtual void IvfSetupUrlFetchCallback(void);
	BOOL HasAnchors(void) { return (m_pAnchorList != NULL); }
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:

// Generated message map functions
protected:
	//{{AFX_MSG(CDriveTrainDoc)
	afx_msg void OnFileImport();
	afx_msg void OnUpdateFileImport(CCmdUI *pCmdUI);
	afx_msg void OnUpdateViewpoints(CCmdUI *pCmdUI);
	afx_msg void OnFileSaveAs();
	afx_msg void OnFileReload();
	afx_msg void OnUpdateFileReload(CCmdUI *pCmdUI);
	afx_msg void OnTrainMode();
	afx_msg void OnDonePosing();
	afx_msg void OnUpdateDonePosing(CCmdUI* pCmdUI);
	afx_msg void OnUpdateTrainMode(CCmdUI* pCmdUI);

	afx_msg void OnMapOpen();
	afx_msg void OnCollectData();
	afx_msg void OnUpdateCollectData(CCmdUI* pCmdUI);
	afx_msg void OnMapClear();
	afx_msg void OnCollectSequence();
	afx_msg void OnUpdateCollectSequence(CCmdUI* pCmdUI);
	afx_msg void OnMapNew();
	afx_msg void OnUpdateMapNew(CCmdUI* pCmdUI);
	//afx_msg void OnYuriOpen();
	//afx_msg void OnPlayYuri();
	//afx_msg void OnUpdatePlayYuri(CCmdUI* pCmdUI);
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()

private:
	//int YuriSeqM;
	//BOOL YuriSeqMade;
	//double** YuriSequence;
	//double slider_vals[10];
	//Model* m_model;
	//CSplineDialog* m_SplineDialog;
	//BOOL yuri_loaded;
	int created_maps;
	BOOL InTrakInstalled;
	SoNodeSensor* InTrakXformSensor;
	int CurrentState;	
	int NumberOfTicks;
	int SequenceWaitTicks;
	int SequenceTicks;
	int MaxNumberOfTicks;
	SbVec3f InTrakTrans1, InTrakTrans2;
	SbRotation InTrakRot1, InTrakRot2;
	Quadraped *q;
	QuadrapedMap *qMap;
	InsideTrakTransform *InTrakXform1, *InTrakXform2;
	M<double> input, lastInput, snapshot, pdfInput;
	M<double> inputSeq;
	GenMap* listOfmaps[NUM_MAPS];
	int currentMapNumber;

	//Mixture pdf[NUM_MAPS];
	double pmap[NUM_MAPS];

	M<double> collectedData;
	int nextCollectedData;

	//M<double> yuriInitValues;

	void OnMapSaveAs();
	void OnMapsSaveCurrentMap();
	void LoadMap(CString); 
	void updateMapNumber();
	void tick();

public:
	CDriveTrainDialog *dialog;
};

